searchData={"content_type":"text/markdown","items":[{"doc":"Rollex is an elixir application that lets you evaluate dice rolls with simple arithmetic operators.\nThe API attempts to follow common, de facto standards found in other dice rolling applications (e.g. VTTs)\nand libraries as much as possible for familiarity and ease of use.\n\n* The operators supported are `+, -, /, *`.\n* Grouping is via parentheses\n* Polyhedral dice are designated using the ` d ` format (Ex. 20d8 or D100).\n  The following operations are supported to filter such rolls:\n    * `<`: consider rolls less than the target number to be successes (`3d10<4`)\n    * `>`: consider rolls greater than the target number to be successes (`3d10>4`)\n    * `=`: consider rolls equal to the target number to be successes (`1d6=6`)\n    * `=[low..high]`: consider rolls between `low` and `high` to be successes (`1d6=[1..2]`)\n    * `=[x,y,..]`: consider rolls in the list of numbers to be successes (`1d6=[1,3,5]` would succeed on odd numbers only)\n    * `o`: only odd numbers are considered succcesses\n    * `e`: only even numbers are considered succcesses\n    * `k` and `kh`: keeps the highest N dice in the pool (`4d6k3` or `4d6kh3`)\n    * `kl`: keeps the lowest N dice in the pool (`4d6kl2`)\n    * `dh`: drops the highest N dice in the pool (`4d6dh2`)\n    * `d` and `dl`: drops the lower N dice in the pool (`4d6d1` or `4d6dl1`)\n    * `r<`, `r>`, and `r=` reroll dice that are less than, greater then, or equal to a number (or a range in the case of equality)\n* [Fudge dice](https://en.wikipedia.org/wiki/Fudge_(role-playing_game_system)) are designated using ' df',\n  The starting state may optionally be provided with a one-letter code for: (t)errible, (p)oor, (m)ediocre, (f)air, (g)ood, g(r)eat, (s)superb\n  If no starting state is defined, it assumed to start at the neutral \"fair\" state.\n\n\nWhen a roll is evaluated, Rollex returns a tagged success tuple (`{:ok | :error, term}`). On success, the `term` is a map that contains a `tokens` and a `totals` field. The `totals` field contains information such as the sum total (arithmetic) and the number of successes, while the tokens field contains each token and their respect results.\n\nTokens with the `is_dice: true` field represent \"rollable\" dice. This can be used to, for example, filter the results for dice that were rolled and show the user a representation of those dice being rolled.\n\nExample:\n\n```\niex> Rollex.roll(\"1d6+1d4\")\n{:ok,\n  %{\n    tokens: [\n      %Rollex.Tokens.RegularDice{\n        arithmetic: 3,\n        is_dice: true,\n        operation: nil,\n        quantity: 1,\n        regex: ~r/A(*)[dD](+)/,\n        rejected_rolls: [],\n        sides: 6,\n        valid_rolls: [3]\n      },\n      %Rollex.Tokens.Addition{regex: ~r/A+/},\n      %Rollex.Tokens.RegularDice{\n        arithmetic: 1,\n        is_dice: true,\n        operation: nil,\n        quantity: 1,\n        regex: ~r/A(*)[dD](+)/,\n        rejected_rolls: [],\n        sides: 4,\n        valid_rolls: [1]\n      },\n      %Rollex.Tokens.End{regex: ~r/Az/}\n    ],\n    totals: %{arithmetic: 4, successes: 2}\n  }\n}\n```\n\nDice expressions may also be precompiled using `compile/2`. The resulting struct can then be passed\nto `evaluate/1` as many times as desired. These are therefore all equivalent:\n\n```\nRollex.roll(\"1d6\")\nRollex.compile(\"1d6\") |> Rollex.evaluate()\n\ndice = Rollex.compile(\"1d6\")\nRollex.evaluate(dice)\nRollex.evaluate(dice)\n```\n\nA map containing the odds for all possible results of a given dice roll can be generated by passing a\ncompiled roll to `histogram/1`:\n\n```\niex> Rollex.compile(\"2d6\") |> Rollex.histogram()\n{:ok,\n  %{\n    2 => 2.778,\n    3 => 5.556,\n    4 => 8.333,\n    5 => 11.111,\n    6 => 13.889,\n    7 => 16.667,\n    8 => 13.889,\n    9 => 11.111,\n    10 => 8.333,\n    11 => 5.556,\n    12 => 2.778\n  }\n}\n```","ref":"Rollex.html","title":"Rollex","type":"module"},{"doc":"Compiles a string representing a die roll into a form suitable for passing into evaluate/1","ref":"Rollex.html#compile/2","title":"Rollex.compile/2","type":"function"},{"doc":"","ref":"Rollex.html#default_tokens/0","title":"Rollex.default_tokens/0","type":"function"},{"doc":"Runs a compiled expression, producing a result","ref":"Rollex.html#evaluate/1","title":"Rollex.evaluate/1","type":"function"},{"doc":"","ref":"Rollex.html#histogram/1","title":"Rollex.histogram/1","type":"function"},{"doc":"","ref":"Rollex.html#max/1","title":"Rollex.max/1","type":"function"},{"doc":"","ref":"Rollex.html#min/1","title":"Rollex.min/1","type":"function"},{"doc":"","ref":"Rollex.html#range/1","title":"Rollex.range/1","type":"function"},{"doc":"Takes a string representing a die roll and returns the result","ref":"Rollex.html#roll/2","title":"Rollex.roll/2","type":"function"},{"doc":"","ref":"Rollex.html#t:diceType/0","title":"Rollex.diceType/0","type":"type"},{"doc":"","ref":"Rollex.html#t:diceTypes/0","title":"Rollex.diceTypes/0","type":"type"},{"doc":"","ref":"Rollex.html#t:evaluatedRoll/0","title":"Rollex.evaluatedRoll/0","type":"type"},{"doc":"","ref":"Rollex.html#t:rollResult/0","title":"Rollex.rollResult/0","type":"type"},{"doc":"","ref":"Rollex.html#t:t/0","title":"Rollex.t/0","type":"type"},{"doc":"","ref":"Rollex.html#t:token/0","title":"Rollex.token/0","type":"type"},{"doc":"","ref":"Rollex.html#t:tokens/0","title":"Rollex.tokens/0","type":"type"},{"doc":"","ref":"Rollex.html#t:totals/0","title":"Rollex.totals/0","type":"type"},{"doc":"","ref":"Rollex.Dice.html","title":"Rollex.Dice","type":"protocol"},{"doc":"","ref":"Rollex.Dice.html#max/1","title":"Rollex.Dice.max/1","type":"function"},{"doc":"","ref":"Rollex.Dice.html#min/1","title":"Rollex.Dice.min/1","type":"function"},{"doc":"","ref":"Rollex.Dice.html#roll/1","title":"Rollex.Dice.roll/1","type":"function"},{"doc":"","ref":"Rollex.Dice.html#t:t/0","title":"Rollex.Dice.t/0","type":"type"},{"doc":"Support for operations that can be applied to dice rolls including:\n\n* odd only (`o`)\n* even oly (`e`)\n* keep high (`kh#, k#`)\n* keep low (`kl#`)\n* drop high (`dh#`)\n* drop low (`dl#`)\n* less than (`<#`)\n* greater than (`>#`)\n* equal to (`=#`, `=[#, #, #..]`, `=[#..#]`)\n* reroll lower than (`r<#`)\n* reroll higher than (`r>#`)\n* reroll equal to (`r=#`, `r=[#, #, #..]`, `r=[#..#]`)\n* explode (`!`)","ref":"Rollex.Dice.Operations.html","title":"Rollex.Dice.Operations","type":"module"},{"doc":"Applies an operation returned by `parse/1` to a list of numbers representing dice rolls","ref":"Rollex.Dice.Operations.html#apply/2","title":"Rollex.Dice.Operations.apply/2","type":"function"},{"doc":"Parse operations from a provided string","ref":"Rollex.Dice.Operations.html#parse/1","title":"Rollex.Dice.Operations.parse/1","type":"function"},{"doc":"","ref":"Rollex.Dice.Operations.html#t:operable_token/0","title":"Rollex.Dice.Operations.operable_token/0","type":"type"},{"doc":"","ref":"Rollex.Dice.Operations.html#t:operation/0","title":"Rollex.Dice.Operations.operation/0","type":"type"},{"doc":"","ref":"Rollex.Dice.Operations.html#t:operation_type/0","title":"Rollex.Dice.Operations.operation_type/0","type":"type"},{"doc":"This module provides functions to calculate various outcome distributions\nof sets of dice. This includes minimums, maximums, and histograms.","ref":"Rollex.Distribution.html","title":"Rollex.Distribution","type":"module"},{"doc":"","ref":"Rollex.Distribution.html#histogram/3","title":"Rollex.Distribution.histogram/3","type":"function"},{"doc":"Calculates the maximum value possible from a roll definition","ref":"Rollex.Distribution.html#max/1","title":"Rollex.Distribution.max/1","type":"function"},{"doc":"Calculates the minimum value possible from a roll definition","ref":"Rollex.Distribution.html#min/1","title":"Rollex.Distribution.min/1","type":"function"},{"doc":"Calculates the minimum and maximum value possible from a roll definition","ref":"Rollex.Distribution.html#range/1","title":"Rollex.Distribution.range/1","type":"function"},{"doc":"Takes histogram, a translation magnitude, and a 3-arity functions to apply to entries in\nthe histogram, returning a translated histogram. Useful for performing arithmetic on histograms\nallowing for the representation of things such as \"1d8+2\"","ref":"Rollex.Distribution.html#translate_histogram/3","title":"Rollex.Distribution.translate_histogram/3","type":"function"},{"doc":"Takes two histograms and combines them into one","ref":"Rollex.Distribution.html#zip_histograms/2","title":"Rollex.Distribution.zip_histograms/2","type":"function"},{"doc":"","ref":"Rollex.Histogram.html","title":"Rollex.Histogram","type":"protocol"},{"doc":"Returns a map containing the odds for every possible result for a token","ref":"Rollex.Histogram.html#evaluate/2","title":"Rollex.Histogram.evaluate/2","type":"function"},{"doc":"","ref":"Rollex.Histogram.html#t:histogram/0","title":"Rollex.Histogram.histogram/0","type":"type"},{"doc":"","ref":"Rollex.Histogram.html#t:t/0","title":"Rollex.Histogram.t/0","type":"type"},{"doc":"","ref":"Rollex.Token.html","title":"Rollex.Token","type":"protocol"},{"doc":"Create a token from the passed-in string representing a roll expression.\nReturns a token and the number of bytes consumed in doing so.","ref":"Rollex.Token.html#create/3","title":"Rollex.Token.create/3","type":"function"},{"doc":"The left binding power (lbp), or the precedence in infix form for operator precedence.\nHigher lbp takes precedence over lower","ref":"Rollex.Token.html#lbp/1","title":"Rollex.Token.lbp/1","type":"function"},{"doc":"Left denotation (led): take action when this token is encountered within an expression,\ne.g. infix and postfix operations","ref":"Rollex.Token.html#led/3","title":"Rollex.Token.led/3","type":"function"},{"doc":"Null denotation (nud): the action take when this token is at the start of an expression,\ne.g. prefix operations","ref":"Rollex.Token.html#nud/2","title":"Rollex.Token.nud/2","type":"function"},{"doc":"","ref":"Rollex.Token.html#t:t/0","title":"Rollex.Token.t/0","type":"type"},{"doc":"","ref":"Rollex.Tokens.FudgeDice.html","title":"Rollex.Tokens.FudgeDice","type":"module"},{"doc":"","ref":"Rollex.Tokens.FudgeDice.html#t:t/0","title":"Rollex.Tokens.FudgeDice.t/0","type":"type"},{"doc":"","ref":"Rollex.Utilities.html","title":"Rollex.Utilities","type":"module"},{"doc":"","ref":"Rollex.Utilities.html#is_arithmetic_total?/1","title":"Rollex.Utilities.is_arithmetic_total?/1","type":"macro"},{"doc":"Merges two sets of dice roll results into one","ref":"Rollex.Utilities.html#merge/3","title":"Rollex.Utilities.merge/3","type":"function"},{"doc":"Conveniently parses a float from a string known to contain a number, with rounding","ref":"Rollex.Utilities.html#simple_float_parse/1","title":"Rollex.Utilities.simple_float_parse/1","type":"function"},{"doc":"Conveniently parses an integer from a string known to contain a number","ref":"Rollex.Utilities.html#simple_integer_parse/1","title":"Rollex.Utilities.simple_integer_parse/1","type":"function"},{"doc":"","ref":"Rollex.Utilities.html#t:merge_operation/0","title":"Rollex.Utilities.merge_operation/0","type":"type"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}